/*
  SD card read/write

 * SD card attached to SPI bus as follows:
 ** MOSI - pin 11
 ** MISO - pin 12
 ** CLK - pin 13
 ** CS - pin 4 (for MKRZero SD: SDCARD_SS_PIN)

 */

#include <VarioSettings.h>
#include <Arduino.h>

boolean VarioSettings::initSettings() {
    if (!SD.begin(SDCARD_CS_PIN)) {
      Serial.println("initialization failed!");
      return false;
    }
  return true;
}

boolean VarioSettings::readSDSettings(){
  char character;
  String settingName;
  String settingValue;
  myFile = SD.open(FileName);
  if (myFile) {
    while (myFile.available()) {
      character = myFile.read();
      while((myFile.available()) && (character != '[')){
        character = myFile.read();
      }
      character = myFile.read();
      while((myFile.available()) && (character != '=')){
        settingName = settingName + character;
        character = myFile.read();
      }
      character = myFile.read();
      while((myFile.available()) && (character != ']')){
        settingValue = settingValue + character;
        character = myFile.read();
      }
      
      if(character == ']'){
 
        /*
        //Debuuging Printing
        Serial.print("Name:");
        Serial.println(settingName);
        Serial.print("Value :");
        Serial.println(settingValue);
        */

        // Apply the value to the parameter
        applySetting(settingName,settingValue);
        // Reset Strings
        settingName = "";
        settingValue = "";
      }
    }
 
    // close the file:
    myFile.close();
	return true;
  } else {
   // if the file didn't open, print an error:
   Serial.println("error opening settings.txt");
   return false;
  }
}
 
 /* Apply the value to the parameter by searching for the parameter name
 Using String.toInt(); for Integers
 toFloat(string); for Float
 toBoolean(string); for Boolean
 toLong(string); for Long
 */
 void VarioSettings::applySetting(String settingName, String settingValue) {
   if(settingName == "exINT") {
     exINT=settingValue.toInt();
   }
   else if(settingName == "exFloat") {
     exFloat=toFloat(settingValue);
   }
   else if(settingName == "exBoolean") {
     exBoolean=toBoolean(settingValue);
   }
   else if(settingName == "exLong") {
     exLong=toLong(settingValue);
   }
   else if (settingName == "VARIOMETER_PILOT_NAME") {
//       Serial.println("Model du vario :" + settingValue);
       VARIOMETER_PILOT_NAME=settingValue;
   }
    else if(settingName == "VARIOMETER_GLIDER_NAME") {
     VARIOMETER_GLIDER_NAME = settingValue;
   }
   else if(settingName == "VARIOMETER_TIME_ZONE") {
     VARIOMETER_TIME_ZONE=settingValue.toInt();
   }
   else {
       
   }  

   uint8_t VARIOMETER_BEEP_VOLUME = 3;

  /* The variometer react like this according to vertical speed in m/s :        */
  /* (near climbing beep is not enabled by default)                             */
  /*                                                                            */
  /* <--LOW-BEEP--|------SILENT------|--NEAR-CLIMBING-BEEP--|--CLIMBING-BEEP--> */
  /*              |                  |                      |                   */
  /*           SINKING         CLIMBING-SENSITIVITY      CLIMBING               */
  float VARIOMETER_SINKING_THRESHOLD =-2.0;
  float VARIOMETER_CLIMBING_THRESHOLD=0.2;
  float VARIOMETER_NEAR_CLIMBING_SENSITIVITY=0.5; 
  
  /* The near climbing alarm : signal that you enter or exit the near climbing zone */
  /* The near climbing beep : beep when you are in near climbing zone               */
  boolean VARIOMETER_ENABLE_NEAR_CLIMBING_ALARM = false;
  boolean VARIOMETER_ENABLE_NEAR_CLIMBING_BEEP  = false;

  /********************/
  /* Measure behavior */
  /********************/

  /* Flight start detection conditions :                      */
  /* -> Minimum time after poweron in milliseconds            */
  /* -> Minimum vertical velocity in m/s (low/high threshold) */
  /* -> Minimum ground speed in km/h                          */
  long FLIGHT_START_MIN_TIMESTAMP = 15000;
  float FLIGHT_START_VARIO_LOW_THRESHOLD = (-0.5);
  float FLIGHT_START_VARIO_HIGH_THRESHOLD = 0.5;
  float FLIGHT_START_MIN_SPEED = 8.0;

  /* GPS track recording on SD card starting condition :  */ 
  /* -> As soon as possible (GPS fix)                     */
  /* -> When flight start is detected                     */
   boolean VARIOMETER_RECORD_WHEN_FLIGHT_START = true;

  /* What type of vario NMEA sentence is sent by bluetooth. */
  /* Possible values are :                                  */
  /*  - VARIOMETER_SENT_LXNAV_SENTENCE                      */
  /*  - VARIOMETER_SENT_LK8000_SENTENCE                     */
  boolean VARIOMETER_SENT_LXNAV_SENTENCE = true;

  /* Alarm */
  /* Alarm SDCARD not insert */
  boolean ALARM_SDCARD = true;
  /* Alarm GPS Fix */
  boolean ALARM_GPSFIX = true;
  /* Alarm Fly begin */
  boolean ALARM_FLYBEGIN = true;
   
}
 
 // converting string to Float
float VarioSettings::toFloat(String settingValue){
   char floatbuf[settingValue.length()+1];
   settingValue.toCharArray(floatbuf, sizeof(floatbuf));
   float f = atof(floatbuf);
   return f;
}
 
long VarioSettings::toLong(String settingValue){
   char longbuf[settingValue.length()+1];
   settingValue.toCharArray(longbuf, sizeof(longbuf));
   long l = atol(longbuf);
   return l;
}
 
 
 // Converting String to integer and then to boolean
 // 1 = true
 // 0 = false
boolean VarioSettings::toBoolean(String settingValue) {
  if(settingValue.toInt()==1){
    return true;
  } else {
    return false;
  }
}
 
 // Writes A Configuration file DEBBUG
void VarioSettings::writeSDSettings() {
  // Delete the old One
  SD.remove(FileName);
  // Create new one
  myFile = SD.open(FileName, FILE_WRITE);
  // writing in the file works just like regular print()/println() function
  myFile.print("[");
  myFile.print("exINT=");
  myFile.print(exINT);
  myFile.println("]");
  myFile.print("[");
  myFile.print("exFloat=");
  myFile.print(exFloat,5);
  myFile.println("]");
  myFile.print("[");
  myFile.print("exBoolean=");
  myFile.print(exBoolean);
  myFile.println("]");
  myFile.print("[");
  myFile.print("exLong=");
  myFile.print(exLong);
  myFile.println("]");
  // close the file:
  myFile.close();
  //Serial.println("Writing done.");
}

